@@ -1,4 +1,4 @@
-use std::collections::HashMap;
+use crate::{all_imports::Shape, hashmaps::FxHashMap as HashMap};
 
 use num_bigint::BigUint;
 
@@ -13,19 +13,12 @@ use super::{
     deep_rewrite::deep_heuristic_nest_adds,
     deep_rewrite::{compiler_simp, deep_push_down_index},
     flat_concat,
-    prelude::*,
-    scheduled_execution::evaluate_schedule_adjust_numerical_scale,
     scheduled_execution::{
-        circuit_to_schedule, circuit_to_schedule_naive_toposort, evaluate_schedule, Schedule,
-        SchedulingError,
+        circuit_to_schedule, circuit_to_schedule_naive_toposort, Schedule, SchedulingError,
     },
     CircuitNode, CircuitRc, HashBytes,
 };
-use crate::{
-    py_types::{Tensor, UN_FLAT_CONCAT},
-    timed,
-    util::apply_fn_until_same,
-};
+use crate::{py_types::Tensor, timed, util::apply_fn_until_same};
 
 // a lot of the function boundries in this are there because it would be too hard to optimize things jointly
 // not because we don't need to optimize things jointly
@@ -228,10 +221,10 @@ pub fn optimize_to_schedule(
     let optimized_circuit = timed!(optimize_circuit(circuit, &mut context), 10, verbose >= 1);
     // return evaluate_fn(&*optimized_circuit).unwrap();
     let schedule = if settings.scheduling_naive {
-        circuit_to_schedule_naive_toposort(optimized_circuit.clone())
+        circuit_to_schedule_naive_toposort(optimized_circuit)
     } else {
         timed!(
-            circuit_to_schedule(optimized_circuit.clone(), settings),
+            circuit_to_schedule(optimized_circuit, settings),
             10,
             verbose >= 1
         )?
@@ -278,7 +271,7 @@ pub fn optimize_to_schedule_many(
         circuits
             .iter()
             .map(|x| x.info().shape.clone())
-            .collect::<Vec<Vec<usize>>>(),
+            .collect::<Vec<Shape>>(),
     );
     Ok(schedule)
 }
