@@ -1,17 +1,17 @@
+use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
+use crate::smallvec::Sv;
+use crate::sv;
 use crate::{
     all_imports::TorchDeviceDtypeOp,
     new_rc_unwrap,
     py_types::{PY_EINSUM, PY_MAKE_DIAGONAL},
     pyo3_prelude::*,
 };
-use itertools::zip;
+use itertools::{zip, Itertools};
 use macro_rules_attribute::apply;
 use num_bigint::BigUint;
 use pyo3::types::IntoPyDict;
-use std::{
-    cmp::Ordering,
-    collections::{HashMap, HashSet},
-};
+use std::cmp::Ordering;
 
 use super::{
     algebraic_rewrite::{add_deduplicate, add_flatten_once, remove_add_few_input},
@@ -31,13 +31,13 @@ use crate::{
         broadcast_shapes, Slice, TensorAxisIndex, TensorIndex, TensorInvariantError,
         TorchDeviceDtype,
     },
-    util::{dedup_with_order, dict_to_list},
+    util::dict_to_list,
     util::{hashmap_collect_except_duplicates, EinInt},
 };
 
-// make me small vec for perf
 // we could use u64 instead of u8 if we wanted.
-pub type EinsumAxes = Vec<EinInt>;
+// this is u8, so we want to use multiple of 8 inline so we don't pad
+pub type EinsumAxes = Sv<[EinInt; 8]>;
 
 /// our Einsum supports diag output
 /// this means `a->aa` is valid and produces a tensor with the input on the diagonal
@@ -135,14 +135,14 @@ impl CircuitNode for Einsum {
                     .extract::<Tensor>(py)
                     .unwrap());
             }
-            let out_axes_deduped = dedup_with_order(&self.out_axes);
+            let out_axes_deduped: EinsumAxes = self.out_axes.iter().unique().cloned().collect();
             let result_non_diag: Tensor =
                 PY_EINSUM
                     .call(
                         py,
                         (
                             zip(tensors.iter().cloned(), self.input_axes().cloned())
-                                .collect::<Vec<(Tensor, Vec<EinInt>)>>(),
+                                .collect::<Vec<(Tensor, EinsumAxes)>>(),
                             out_axes_deduped.clone(),
                         ),
                         None,
@@ -263,11 +263,11 @@ impl Einsum {
 
     pub fn scalar_mul(node: &CircuitRc, scalar: f64, name: Option<String>) -> Self {
         let name_definite = name.unwrap_or_else(|| "scalar_mul".to_owned());
-        let axes: Vec<u8> = (0..node.info().rank()).map(|x| x as u8).collect();
+        let axes: EinsumAxes = (0..node.info().rank()).map(|x| x as u8).collect();
         Einsum::try_new(
             vec![
                 (node.clone(), axes.clone()),
-                (ScalarConstant::new(scalar, vec![], None).rc(), vec![]),
+                (ScalarConstant::new(scalar, sv![], None).rc(), sv![]),
             ],
             axes,
             Some(name_definite),
@@ -367,14 +367,14 @@ impl Einsum {
     }
 
     #[staticmethod]
-    pub fn new_diag(node: CircuitRc, ints: Vec<EinInt>, name: Option<String>) -> Self {
-        let deduped = dedup_with_order(&ints);
+    pub fn new_diag(node: CircuitRc, ints: EinsumAxes, name: Option<String>) -> Self {
+        let deduped = ints.iter().unique().cloned().collect();
         Einsum::try_new(vec![(node, deduped)], ints, name).unwrap()
     }
 
     #[staticmethod]
-    pub fn new_trace(node: CircuitRc, ints: Vec<EinInt>, name: Option<String>) -> Self {
-        let deduped = dedup_with_order(&ints);
+    pub fn new_trace(node: CircuitRc, ints: EinsumAxes, name: Option<String>) -> Self {
+        let deduped = ints.iter().unique().cloned().collect();
         Einsum::try_new(vec![(node, ints)], deduped, name).unwrap()
     }
 
@@ -390,7 +390,7 @@ impl Einsum {
 
     #[getter]
     fn out_axes(&self) -> PyEinsumAxes {
-        PyEinsumAxes(self.out_axes.to_vec())
+        PyEinsumAxes(self.out_axes.clone())
     }
 
     #[getter]
@@ -477,7 +477,7 @@ impl Add {
         !self
             .nodes
             .iter()
-            .all(|node| node.info().shape[..] == self.info().shape)
+            .all(|node| node.info().shape == self.info().shape)
     }
 
     pub fn nodes_and_rank_differences(&self) -> Vec<(CircuitRc, usize)> {
@@ -845,25 +845,6 @@ impl Index {
             ]),
             _ => {}
         }
-        for (idx, l) in zip(&index.0, &node.info().shape) {
-            match idx {
-                TensorAxisIndex::Single(i) => {
-                    if *i < -(*l as i64) || i >= &(*l as i64) {
-                        return Err(CircuitConstructionError::IndexOutOfBounds {
-                            index: index.clone(),
-                            shape: node.info().shape.clone(),
-                        });
-                    }
-                }
-                TensorAxisIndex::Slice(slice) => {
-                    // panic on invalid
-                    slice.stop_u(*l);
-                    slice.start_u(*l);
-                }
-
-                _ => {}
-            }
-        }
         let index = TensorIndex(
             index
                 .0
