@@ -1,8 +1,8 @@
-use crate::{new_rc, pyo3_prelude::*};
+use crate::hashmaps::FxHashMap as HashMap;
+use crate::{new_rc, pyo3_prelude::*, sv};
 use macro_rules_attribute::apply;
 use num_bigint::BigUint;
 use pyo3::types::{IntoPyDict, PyTuple};
-use std::collections::HashMap;
 use uuid::Uuid;
 
 use super::{
@@ -124,7 +124,7 @@ impl ArrayConstant {
 
     #[staticmethod]
     pub fn randn_named(
-        shape: Vec<usize>,
+        shape: Shape,
         name: Option<String>,
         device_dtype: TorchDeviceDtypeOp,
     ) -> Self {
@@ -155,13 +155,13 @@ impl ArrayConstant {
 
     #[staticmethod]
     #[args(args = "*")]
-    pub fn randn(args: Vec<usize>) -> Self {
+    pub fn randn(args: Shape) -> Self {
         ArrayConstant::randn_named(args, None, TorchDeviceDtypeOp::default())
     }
 
     #[staticmethod]
     pub fn randn_seeded(
-        shape: Vec<usize>,
+        shape: Shape,
         name: Option<String>,
         device_dtype: TorchDeviceDtypeOp,
         seed: usize,
@@ -321,7 +321,7 @@ impl ScalarConstant {
 impl ScalarConstant {
     #[cfg(feature = "real-pyo3")]
     #[new]
-    #[args(shape = "vec![]", name = "\"ScalarConstant\".to_owned()")]
+    #[args(shape = "sv![]", name = "\"ScalarConstant\".to_owned()")]
     fn py_new(value: f64, shape: Shape, name: Option<String>) -> PyClassInitializer<Self> {
         Self::new(value, shape, name).into_init()
     }
@@ -337,6 +337,6 @@ impl ScalarConstant {
 
 #[test]
 fn test_nrc() {
-    let ex = ScalarConstant::nrc(0.0, vec![1, 2], None);
+    let ex = ScalarConstant::nrc(0.0, sv![1, 2], None);
     ex.compiler_print();
 }
