@@ -1,4 +1,5 @@
-use std::collections::{HashMap, HashSet};
+use crate::circuit::scatter_rewrite::concat_to_scatter;
+use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 
 use crate::pyo3_prelude::*;
 use num_bigint::BigUint;
@@ -7,8 +8,8 @@ use crate::util::{apply_fn_until_same, mapping_until_end, AsOp};
 
 use super::{
     algebraic_rewrite::*,
-    canonicalize::{canonicalize_node, deep_canonicalize},
-    circuit_optimizer::{OptimizationCache, OptimizationContext},
+    canonicalize::deep_canonicalize,
+    circuit_optimizer::OptimizationContext,
     concat_rewrite::add_pull_concat,
     concat_rewrite::{
         concat_drop_size_zero, concat_fuse, einsum_pull_concat, generalfunction_pull_concat,
@@ -112,6 +113,7 @@ pub fn compiler_simp_step(circuit: &Circuit) -> Option<CircuitRc> {
                 f_wrap!(&concat_drop_size_zero),
                 f_wrap!(&concat_fuse),
                 f_wrap!(&concat_repeat_to_rearrange),
+                f_wrap!(&concat_to_scatter),
             ];
             simp(node, fns)
         }
@@ -258,7 +260,7 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
                 for cand_sup in &seen_adds {
                     if cand_sub != cand_sup {
                         if let Some(new) = extract_add(cand_sup, cand_sub) {
-                            return Some((cand_sup.clone(), new.clone()));
+                            return Some((cand_sup.clone(), new));
                         }
                     }
                 }
@@ -287,7 +289,7 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
 pub fn add_nest_ltr(add: &Add) -> Add {
     let (l, r) = add.nodes.split_at(2.min(add.nodes.len()));
     let base = Add::try_new(l.to_vec(), None).unwrap();
-    r.into_iter().fold(base, |acc, x| {
+    r.iter().fold(base, |acc, x| {
         Add::try_new(vec![acc.rc(), x.clone()], None).unwrap()
     })
 }
