@@ -1,6 +1,6 @@
-use crate::all_imports::TorchDeviceDtypeOp;
+use crate::hashmaps::FxHashMap as HashMap;
 use crate::pyo3_prelude::*;
-use std::{collections::HashMap, iter::zip};
+use std::iter::zip;
 
 use super::{prelude::*, CachedCircuitInfo, HashBytes};
 use crate::py_types::{GENERALFUNCTIONS, PY_NONE};
@@ -345,39 +345,3 @@ impl GeneralFunction {
         GeneralFunction::try_new(nodes, spec.clone(), name)
     }
 }
-
-#[test]
-#[ignore] // needs torch which ci rust test doesn't have...should add that
-fn test_generalfuction() {
-    use super::{Add, ArrayConstant, Einsum};
-    use crate::circuit::circuit_optimizer::optimize_and_evaluate;
-
-    pyo3::prepare_freethreaded_python();
-    let circuit = Einsum::nrc(
-        vec![(
-            Add::nrc(
-                vec![
-                    ArrayConstant::randn_named(vec![2, 3, 4], None, TorchDeviceDtypeOp::default())
-                        .rc(),
-                    GeneralFunction::new_by_name(
-                        vec![ArrayConstant::randn_named(
-                            vec![2, 3, 4],
-                            None,
-                            TorchDeviceDtypeOp::default(),
-                        )
-                        .rc()],
-                        "sigmoid".to_owned(),
-                        None,
-                    )
-                    .unwrap()
-                    .rc(),
-                ],
-                None,
-            ),
-            vec![0, 1, 2],
-        )],
-        vec![0, 1],
-        None,
-    );
-    optimize_and_evaluate(circuit, Default::default()).unwrap();
-}
