@@ -1,11 +1,11 @@
 #![allow(clippy::borrow_deref_ref)]
+use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 use crate::py_types::PY_MAKE_BYTES;
 use crate::{pycall, pyo3_prelude::*};
 use num_bigint::BigUint;
 use pyo3::{exceptions, pyclass::CompareOp};
 use std::hash::Hash;
 use std::{
-    collections::{HashMap, HashSet},
     iter::zip,
     ops::{Deref, DerefMut},
     rc::Rc,
@@ -228,7 +228,11 @@ pub trait CircuitNode: CircuitNodePrivate {
         self.info_mut().shape = shape.clone(); // set shape so methods to compute other info can use it
 
         let mut hasher = self.compute_hash();
-        let named_axes = self.compute_named_axes();
+        let named_axes = if self.children().any(|x| !x.info().named_axes.is_empty()) {
+            self.compute_named_axes()
+        } else {
+            vec![]
+        };
         hasher.update(self.name().unwrap_or("").as_bytes());
         hasher.update(&self.node_type_uuid());
         hasher.update(uuid!("025e9af4-1366-4211-aa5f-7c28fc6cdf9f").as_bytes());
@@ -1024,7 +1028,7 @@ where
     fn recurse(circ: &Circuit) -> CircuitRc {
         let old_children: Vec<CircuitRc> = circ.children().collect();
         let new_children = old_children.iter().map(|x| &***x).map(recurse).collect();
-        f(&circ, &new_children)
+        f(circ, &new_children)
     }
     recurse(circuit)
 }
@@ -1087,7 +1091,7 @@ where
     let inner_mapped = circuit.map_children_op(|x| deep_map_op_context(x, f, context, self_cache));
     let result = match inner_mapped {
         Some(z) => f(&z, context).or(Some(z.rc())),
-        None => f(&circuit, context),
+        None => f(circuit, context),
     };
     self_cache.insert(circuit.info().hash, result.clone());
     result
@@ -1206,7 +1210,7 @@ pub fn circuit_rust_to_py(circ: CircuitRc) -> PyObject {
 impl PyCircuitBase {
     #[getter]
     fn shape(&self) -> PyShape {
-        PyShape(self.info().shape.to_vec())
+        PyShape(self.info().shape.clone())
     }
 
     #[getter]
