@@ -1,7 +1,8 @@
+use smallvec::Array;
 use uuid::Uuid;
 
 use super::{OpSize, RearrangeSpec, TensorAxisIndex, TensorIndex};
-use crate::{all_imports::TorchDeviceDtypeOp, tensor_util::Shape};
+use crate::{all_imports::TorchDeviceDtypeOp, smallvec::Sv, tensor_util::Shape};
 
 pub trait RustRepr {
     /// Serialize objects to Rust expression source code
@@ -60,6 +61,16 @@ impl<T: RustRepr> RustRepr for Vec<T> {
     }
 }
 
+impl<T: Array> RustRepr for Sv<T>
+where
+    T::Item: RustRepr,
+{
+    fn repr(&self) -> String {
+        let strings: Vec<String> = self.iter().map(|s| s.repr()).collect();
+        format!("sv![{}]", strings.join(","))
+    }
+}
+
 impl RustRepr for RearrangeSpec {
     fn repr(&self) -> String {
         format!(
