@@ -1,7 +1,12 @@
-use std::{
-    collections::{HashMap, HashSet},
-    iter::zip,
+use smallvec::Array;
+
+use crate::{
+    all_imports::Shape,
+    hashmaps::{AHashSet as HashSet, FxHashMap as HashMap},
+    smallvec::Sv,
+    util::cumsum,
 };
+use std::iter::zip;
 
 use super::{
     algebraic_rewrite::{get_removable_axes, remove_axes},
@@ -38,10 +43,13 @@ pub fn scatter_fuse(scatter: &Scatter) -> Option<Scatter> {
     }
 }
 
-pub fn uslice_map_get_index<T: Eq + Hash>(
-    ints: &Vec<T>,
-    int_slices: &HashMap<T, USlice>,
-) -> TensorIndex {
+pub fn uslice_map_get_index<A: Array>(
+    ints: &Sv<A>,
+    int_slices: &HashMap<A::Item, USlice>,
+) -> TensorIndex
+where
+    A::Item: Eq + Hash,
+{
     TensorIndex(
         ints.iter()
             .map(|i| {
@@ -319,7 +327,12 @@ pub fn scatter_pull_removable_axes(scatter: &Scatter) -> Option<Rearrange> {
                 Scatter::try_new(
                     new_inner,
                     TensorIndex(filter_out_idx(&scatter.index.0, &removable_axes)),
-                    filter_out_idx(&scatter.info().shape, &removable_axes),
+                    filter_out_idx(
+                        &scatter.info().shape.iter().cloned().collect::<Vec<_>>(),
+                        &removable_axes,
+                    )
+                    .into_iter()
+                    .collect(),
                     None,
                 )
                 .unwrap()
@@ -338,9 +351,9 @@ pub fn scatter_to_concat(scatter: &Scatter) -> CircuitRc {
     for (i, (idx, l)) in
         zip(scatter.index.all_uslices().unwrap(), &scatter.info().shape).enumerate()
     {
-        let mut lower_pad_shape: Vec<usize> = result.info().shape.clone();
+        let mut lower_pad_shape: Shape = result.info().shape.clone();
         lower_pad_shape[i] = idx.start;
-        let mut upper_pad_shape: Vec<usize> = result.info().shape.clone();
+        let mut upper_pad_shape: Shape = result.info().shape.clone();
         upper_pad_shape[i] = l - idx.stop;
         result = Concat::try_new(
             vec![
@@ -356,3 +369,44 @@ pub fn scatter_to_concat(scatter: &Scatter) -> CircuitRc {
     }
     result
 }
+
+#[pyfunction]
+pub fn concat_to_scatter(concat: &Concat) -> Option<Scatter> {
+    let pre_zeros = concat
+        .nodes
+        .iter()
+        .take_while(|node| {
+            if let Circuit::ScalarConstant(sc) = &****node && sc.value==0.0{
+            true
+        }else{
+            false
+        }
+        })
+        .count();
+    let post_zeros = concat
+        .nodes
+        .iter()
+        .rev()
+        .take_while(|node| {
+            if let Circuit::ScalarConstant(sc) = &****node && sc.value==0.0{
+            true
+        }else{
+            false
+        }
+        })
+        .count();
+    let end = concat.nodes.len() - post_zeros;
+    if pre_zeros == 0 && post_zeros == 0 || (pre_zeros + post_zeros >= concat.nodes.len()) {
+        return None;
+    }
+    let starts = cumsum(&concat.get_sections());
+    let cslice = TensorAxisIndex::new_plain_slice(starts[pre_zeros], starts[end]);
+    let scatter_index = TensorIndex::new_single(cslice, concat.axis, concat.info().rank())
+        .canonicalize(&concat.info().shape);
+    let new_concat = Concat::nrc(
+        concat.nodes[pre_zeros..end].to_vec(),
+        concat.axis,
+        concat.name_cloned(),
+    );
+    Some(Scatter::try_new(new_concat, scatter_index, concat.info().shape.clone(), None).unwrap())
+}
