@@ -1,11 +1,8 @@
-use std::{
-    collections::{HashMap, HashSet},
-    fmt::{self, Display},
-};
+use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
+use std::fmt::{self, Display};
 
 use crate::{
     cached_lambda,
-    circuit::circuit_optimizer::OptimizationContext,
     py_types::{ExtraPySelfOps, Tensor, PY_SCALAR_TO_TENSOR, TENSOR_SCALE, UN_FLAT_CONCAT},
     pycall,
     tensor_util::{Shape, TorchDeviceDtype},
@@ -19,7 +16,7 @@ use super::{
     prelude::*,
     print::oom_fmt,
     scheduling_z3::schedule_dag_strategy_ints,
-    visit_circuit, ArrayConstant, HashBytes, PyHashBytes, ScalarConstant, Symbol,
+    visit_circuit, ArrayConstant, HashBytes, PyHashBytes, ScalarConstant,
 };
 use crate::pyo3_prelude::*;
 use crate::util::filter_to_idx;
@@ -45,7 +42,7 @@ impl IntoPy<PyObject> for Instruction {
 }
 
 #[pyclass(unsendable)]
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PyClassDeriv)]
 pub struct Schedule {
     #[pyo3(get)]
     pub instructions: Vec<Instruction>,
@@ -150,13 +147,13 @@ impl Schedule {
 }
 
 #[pyclass(unsendable)]
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PyClassDeriv)]
 pub struct ScheduleStats {
     #[pyo3(get)]
     max_mem: BigUint,
     #[pyo3(get)]
     constant_mem: BigUint, // this has already been allocated so it can't be over 2^64
-    #[pyo3(get)]
+    // #[pyo3(get)]
     max_circuit_set: HashSet<CircuitRc>,
 }
 
@@ -581,9 +578,7 @@ impl Dag {
                 node
             }
         }
-        let node_to_top: Vec<usize> = (0..self.node_costs.len())
-            .map(|node| top_of_chain(node))
-            .collect();
+        let node_to_top: Vec<usize> = (0..self.node_costs.len()).map(top_of_chain).collect();
         node_to_top
             .into_iter()
             .enumerate()
