@@ -1,5 +1,3 @@
-use std::collections::HashMap;
-
 use crate::pyo3_prelude::*;
 use lazy_static::lazy_static;
 use pyo3::{
@@ -18,8 +16,8 @@ lazy_static! {
         PyModule::from_code(
             py,
             include_str!(concat!(env!("CARGO_MANIFEST_DIR"), "/rust_circuit_type_utils.py")),
-            concat!(env!("CARGO_MANIFEST_DIR"), "/rust_circuit_type_utils.py"),
-            "rust_circuit.rust_circuit_type_utils",
+            "rust_circuit_type_utils.py",
+            "rust_circuit_type_utils",
         )
         .unwrap() // TODO: fail more gracefully?
         .into()
@@ -47,7 +45,7 @@ lazy_static! {
         Python::with_gil(|py| PYTHON_UTILS.getattr(py, "un_flat_concat").unwrap());
     pub static ref TENSOR_SCALE: PyObject =
         Python::with_gil(|py| PYTHON_UTILS.getattr(py, "tensor_scale").unwrap());
-    pub static ref GENERALFUNCTIONS: HashMap<String,PyObject> =
+    pub static ref GENERALFUNCTIONS: std::collections::HashMap<String,PyObject> =
         Python::with_gil(|py| PYTHON_UTILS.getattr(py, "generalfunctions").unwrap().extract(py).unwrap());
     pub static ref PY_NONE: PyObject=
         Python::with_gil(|py| PYTHON_UTILS.getattr(py, "none").unwrap().extract(py).unwrap());
@@ -63,9 +61,9 @@ lazy_static! {
     static ref PYCIRCUIT_MODULE: Py<PyModule> = Python::with_gil(|py| {
         PyModule::from_code(
             py,
-            include_str!(concat!(env!("CARGO_MANIFEST_DIR"), "/py_circuit_interop.py")),
-            concat!(env!("CARGO_MANIFEST_DIR"), "/py_circuit_interop.py"),
-            "rust_circuit.py_circuit_interop",
+            "from interp.circuit import computational_node, constant; import interp.circuit.interop_rust.interop_rust as interop; import einops; import interp.circuit.circuit_compiler.util as circ_compiler_util",
+            "py_circuit_interop.py",
+            "py_circuit_interop",
         )
         .unwrap() // TODO: fail more gracefully?
         .into()
@@ -90,8 +88,8 @@ lazy_static! {
         PyModule::from_code(
             py,
             include_str!(concat!(env!("CARGO_MANIFEST_DIR"), "/tensor_hash.py")),
-            concat!(env!("CARGO_MANIFEST_DIR"), "/tensor_hash.py"),
-            "rust_circuit.tensor_hash",
+            "tensor_hash.py",
+            "tensor_hash",
         )
         .unwrap() // TODO: fail more gracefully?
         .into()
