@@ -1,18 +1,19 @@
+use crate::circuit::EinsumAxes;
+use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 use crate::pyo3_prelude::*;
 use crate::{
     circuit::HashBytes,
     py_types::PY_CIRC_COMPILER_UTIL,
+    smallvec::Sv,
+    sv,
     tensor_util::{Shape, TensorInvariantError},
     util::{dict_to_list, filter_out_idx, is_unique, vec_map_insert, EinInt, ALPHABET},
 };
 use itertools::{izip, Itertools};
 use pyo3::exceptions;
 use pyo3::types::IntoPyDict;
-use std::{
-    collections::{HashMap, HashSet},
-    fmt::Debug,
-    iter::zip,
-};
+use std::{fmt::Debug, iter::zip};
+
 use thiserror::Error;
 use uuid::uuid;
 
@@ -87,18 +88,36 @@ pub fn shape_to_op_shape(shape: &Shape) -> OpShape {
     shape.iter().map(|x| OpSize(*x as u64)).collect()
 }
 
-// Don't allow squeeze!
-#[pyo3::prelude::pyclass]
-#[derive(Debug, Clone, PartialEq, Eq)]
+// RInnerInts distribution is heavy tailed - almost everything's 1, but there might be up to 10.
+// we use multiple of 8 bc they're u8 and we'd just pad if we used less
+pub type RInnerInts = Sv<[EinInt; 8]>;
+pub type RInts = Sv<[RInnerInts; 8]>;
+
+#[pyclass]
+#[derive(Debug, Clone, PartialEq, Eq, PyClassDeriv)]
 pub struct RearrangeSpec {
     #[pyo3(get)]
-    pub input_ints: Vec<Vec<EinInt>>,
+    pub input_ints: RInts,
     #[pyo3(get)]
-    pub output_ints: Vec<Vec<EinInt>>,
+    pub output_ints: RInts,
     #[pyo3(get)]
     pub int_sizes: OpShape,
 }
 
+#[cfg(not(feature = "real-pyo3"))]
+impl<'source> FromPyObject<'source> for RearrangeSpec {
+    fn extract(_: &'source PyAny) -> PyResult<Self> {
+        unimplemented!()
+    }
+}
+
+#[cfg(not(feature = "real-pyo3"))]
+impl IntoPy<PyObject> for RearrangeSpec {
+    fn into_py(self, _: Python<'_>) -> PyObject {
+        unimplemented!()
+    }
+}
+
 /// RearrangeSpec encodes the same thing as a python Einops.repeat string https://einops.rocks/api/repeat/
 ///
 /// Dimension names are encoded as integers, so the einops operation
@@ -115,14 +134,10 @@ pub struct RearrangeSpec {
 ///
 /// Most rearranges don't allocate new memory, including non-flattened repeats (which simply have stride 0).
 /// See Rearrange::intermediate_cost_bound for details.
-#[pyo3::prelude::pymethods]
+#[pymethods]
 impl RearrangeSpec {
     #[new]
-    pub fn new(
-        input_ints: Vec<Vec<EinInt>>,
-        output_ints: Vec<Vec<EinInt>>,
-        int_sizes: OpShape,
-    ) -> Self {
+    pub fn new(input_ints: RInts, output_ints: RInts, int_sizes: OpShape) -> Self {
         // TODO: validation
         RearrangeSpec {
             input_ints,
@@ -135,8 +150,8 @@ impl RearrangeSpec {
     pub fn flatten(rank: usize) -> Self {
         assert!(rank <= u8::MAX as usize + 1);
         RearrangeSpec {
-            input_ints: (0..rank).map(|i| vec![i as u8]).collect(),
-            output_ints: vec![(0..rank as u8).collect()],
+            input_ints: (0..rank).map(|i| sv![i as u8]).collect(),
+            output_ints: sv![(0..rank as u8).collect()],
             int_sizes: vec![OpSize::from(None); rank],
         }
     }
@@ -146,8 +161,8 @@ impl RearrangeSpec {
         let rank = shape.len();
         assert!(rank <= u8::MAX as usize + 1);
         RearrangeSpec {
-            input_ints: vec![(0..rank as u8).collect()],
-            output_ints: (0..rank).map(|i| vec![i as u8]).collect(),
+            input_ints: sv![(0..rank as u8).collect()],
+            output_ints: (0..rank).map(|i| sv![i as u8]).collect(),
             int_sizes: shape_to_op_shape(&shape),
         }
     }
@@ -155,8 +170,8 @@ impl RearrangeSpec {
     #[staticmethod]
     pub fn ident(rank: usize) -> Self {
         RearrangeSpec {
-            input_ints: (0..rank).map(|i| vec![i as u8]).collect(),
-            output_ints: (0..rank).map(|i| vec![i as u8]).collect(),
+            input_ints: (0..rank).map(|i| sv![i as u8]).collect(),
+            output_ints: (0..rank).map(|i| sv![i as u8]).collect(),
             int_sizes: vec![OpSize::from(None); rank],
         }
     }
@@ -181,10 +196,10 @@ impl RearrangeSpec {
         })
     }
     pub fn ints_in_inp(&self) -> HashSet<EinInt> {
-        self.input_ints.iter().flatten().copied().collect()
+        self.ints_in_inp_it().collect()
     }
     pub fn ints_in_out(&self) -> HashSet<EinInt> {
-        self.output_ints.iter().flatten().copied().collect()
+        self.ints_in_out_it().collect()
     }
     pub fn is_permute(&self) -> bool {
         // because there are no squeezes, all we need is no splits/joins and equal input and output rank
@@ -215,8 +230,8 @@ impl RearrangeSpec {
 
     pub fn is_valid(&self) -> bool {
         // check each int appears once in input and output
-        let result = is_unique(&self.input_ints.iter().flatten().collect::<Vec<&EinInt>>()[..])
-            && is_unique(&self.input_ints.iter().flatten().collect::<Vec<&EinInt>>()[..]);
+        let result = is_unique(&self.ints_in_inp_it().collect::<Vec<_>>())
+            && is_unique(&self.ints_in_out_it().collect::<Vec<EinInt>>());
         // check each input is in output
         let ints_in_out = self.ints_in_out();
         let ints_in_inp = self.ints_in_inp();
@@ -224,13 +239,7 @@ impl RearrangeSpec {
 
         // check int_sizes is long enough
         let result = result
-            && self
-                .output_ints
-                .iter()
-                .flatten()
-                .max()
-                .map(|i| *i as i64)
-                .unwrap_or(-1)
+            && ints_in_out.iter().max().map(|&i| i as i64).unwrap_or(-1)
                 < self.int_sizes.len() as i64;
 
         // check each int only in output has size
@@ -302,18 +311,19 @@ impl RearrangeSpec {
     }
     pub fn letter_sizes(&self) -> Vec<(String, u64)> {
         // because to_einops_string uses numerals on all added dims, we only need letter sizes for splits
-        let ints_needing_sizes: HashSet<&EinInt> = self
+        let ints_needing_sizes: HashSet<EinInt> = self
             .input_ints
             .iter()
             .filter(|x| x.len() > 1)
             .flatten()
+            .copied()
             .collect();
         ints_needing_sizes
-            .iter()
+            .into_iter()
             .map(|x| {
-                let size_here = self.int_sizes[**x as usize];
+                let size_here = self.int_sizes[x as usize];
                 assert!(size_here.is_some());
-                (ALPHABET[**x as usize].to_owned(), size_here.0)
+                (ALPHABET[x as usize].to_owned(), size_here.0)
             })
             .collect()
     }
@@ -337,7 +347,7 @@ impl RearrangeSpec {
     fn __repr__(&self) -> String {
         format!("{:?}", self)
     }
-    pub fn shapes(&self) -> Result<(Vec<usize>, Vec<usize>), TensorInvariantError> {
+    pub fn shapes(&self) -> Result<(Shape, Shape), TensorInvariantError> {
         if self.int_sizes.iter().any(|x| x.is_none()) {
             return Err(TensorInvariantError::RearrangeHasWildcardSizes { spec: self.clone() });
         }
@@ -420,7 +430,7 @@ impl RearrangeSpec {
         let mut input_ints = self.input_ints.clone();
         let mut output_ints = self.output_ints.clone();
         if special_case_ones {
-            let drop_einints = |ints_list: Vec<Vec<u8>>| {
+            let drop_einints = |ints_list: RInts| {
                 ints_list
                     .iter()
                     .map(|ints| {
@@ -491,10 +501,10 @@ impl RearrangeSpec {
         let mut new_int_sizes: Vec<u64> = Vec::new();
         let mut already_accounted_shape = vec![false; int_sizes.len()];
 
-        let mut merge_tuples_and_renumber_fn = |ints: Vec<Vec<EinInt>>| {
+        let mut merge_tuples_and_renumber_fn = |ints: RInts| {
             ints.iter()
                 .map(|single_dim| {
-                    let mut new_dim = Vec::new();
+                    let mut new_dim = sv![];
                     for &i in single_dim.iter() {
                         if let Some(&new_i) = map.get(&i) {
                             new_dim.push(new_i);
@@ -514,8 +524,8 @@ impl RearrangeSpec {
                 .collect()
         };
 
-        let input_ints: Vec<Vec<EinInt>> = merge_tuples_and_renumber_fn(input_ints);
-        let output_ints: Vec<Vec<EinInt>> = merge_tuples_and_renumber_fn(output_ints);
+        let input_ints: RInts = merge_tuples_and_renumber_fn(input_ints);
+        let output_ints: RInts = merge_tuples_and_renumber_fn(output_ints);
 
         RearrangeSpec {
             input_ints,
@@ -531,13 +541,7 @@ impl RearrangeSpec {
         &self,
         allow_rust_invalid: bool,
     ) -> Result<RearrangeSpec, TensorInvariantError> {
-        let mut next_int = self
-            .output_ints
-            .iter()
-            .flatten()
-            .max()
-            .map(|i| i + &1)
-            .unwrap_or(0);
+        let mut next_int = self.all_ints().max().map(|i| i + 1).unwrap_or(0);
         let first_int_to_add = next_int;
         let input_ints = self
             .input_ints
@@ -545,20 +549,20 @@ impl RearrangeSpec {
             .map(|ints| {
                 if ints.is_empty() {
                     next_int += 1;
-                    vec![next_int - 1]
+                    sv![next_int - 1]
                 } else {
                     ints.clone()
                 }
             })
             .collect();
         let int_after_input = next_int;
-        let mut output_ints: Vec<Vec<u8>> = self
+        let mut output_ints: RInts = self
             .output_ints
             .iter()
             .map(|ints| {
                 if ints.is_empty() {
                     next_int += 1;
-                    vec![next_int - 1]
+                    sv![next_int - 1]
                 } else {
                     ints.clone()
                 }
@@ -604,11 +608,18 @@ impl RearrangeSpec {
 }
 
 impl RearrangeSpec {
-    pub fn new_canon(
-        input_ints: Vec<Vec<EinInt>>,
-        output_ints: Vec<Vec<EinInt>>,
-        int_sizes: OpShape,
-    ) -> Self {
+    pub fn ints_in_inp_it(&self) -> impl Iterator<Item = EinInt> + '_ {
+        self.input_ints.iter().flatten().copied()
+    }
+    pub fn ints_in_out_it(&self) -> impl Iterator<Item = EinInt> + '_ {
+        self.output_ints.iter().flatten().copied()
+    }
+    /// All integers must occur in the output, so this is valid
+    pub fn all_ints(&self) -> impl Iterator<Item = EinInt> + '_ {
+        self.ints_in_out_it()
+    }
+
+    pub fn new_canon(input_ints: RInts, output_ints: RInts, int_sizes: OpShape) -> Self {
         let result = RearrangeSpec {
             input_ints,
             output_ints,
@@ -678,7 +689,9 @@ impl RearrangeSpec {
     pub fn filter_out_axes_unsafe(&self, out_axes_to_skip: &HashSet<usize>) -> RearrangeSpec {
         RearrangeSpec {
             input_ints: self.input_ints.clone(),
-            output_ints: filter_out_idx(&self.output_ints, out_axes_to_skip),
+            output_ints: filter_out_idx(&self.output_ints, out_axes_to_skip)
+                .into_iter()
+                .collect(),
             int_sizes: self.int_sizes.clone(),
         }
     }
@@ -706,24 +719,26 @@ impl RearrangeSpec {
         RearrangeSpec {
             input_ints: (0..output_shape.len())
                 .filter(|i| !removed_axes.contains(i))
-                .map(|i| vec![i as u8])
+                .map(|i| sv![i as u8])
                 .collect(),
-            output_ints: (0..output_shape.len()).map(|i| vec![i as u8]).collect(),
+            output_ints: (0..output_shape.len()).map(|i| sv![i as u8]).collect(),
             int_sizes: shape_to_op_shape(output_shape),
         }
     }
 
     pub fn expand(
-        input_ints: &Vec<u8>,
-        output_ints: &Vec<u8>,
+        input_ints: &EinsumAxes,
+        output_ints: &EinsumAxes,
         int_sizes: &HashMap<u8, usize>,
     ) -> RearrangeSpec {
         let int_sizes_usize: HashMap<usize, usize> =
             int_sizes.iter().map(|(k, v)| (*k as usize, *v)).collect();
         RearrangeSpec {
-            input_ints: input_ints.iter().map(|i| vec![*i]).collect(),
-            output_ints: output_ints.iter().map(|i| vec![*i]).collect(),
-            int_sizes: shape_to_op_shape(&dict_to_list(&int_sizes_usize, None)),
+            input_ints: input_ints.iter().map(|i| sv![*i]).collect(),
+            output_ints: output_ints.iter().map(|i| sv![*i]).collect(),
+            int_sizes: shape_to_op_shape(
+                &dict_to_list(&int_sizes_usize, None).into_iter().collect(),
+            ),
         }
     }
 
@@ -756,19 +771,19 @@ impl RearrangeSpec {
     /// ```
     /// # use rust_circuit::rearrange_spec::{OpSize, RearrangeSpec};
     /// let inner = RearrangeSpec::new(
-    ///     vec![vec![0], vec![1]],
-    ///     vec![vec![0, 1]],
-    ///     vec![OpSize(2), OpSize(3)],
+    ///     sv![sv![0], sv![1]],
+    ///     sv![sv![0, 1]],
+    ///     sv![OpSize(2), OpSize(3)],
     /// );
     /// let outer = RearrangeSpec::new(
-    ///     vec![vec![0]],
-    ///     vec![vec![0], vec![1]],
-    ///     vec![OpSize(6), OpSize(4)],
+    ///     sv![sv![0]],
+    ///     sv![sv![0], sv![1]],
+    ///     sv![OpSize(6), OpSize(4)],
     /// );
     /// let fused = RearrangeSpec::new(
-    ///     vec![vec![0], vec![1]],
-    ///     vec![vec![0, 1], vec![2]],
-    ///     vec![OpSize(2), OpSize(3), OpSize(4)],
+    ///     sv![sv![0], sv![1]],
+    ///     sv![sv![0, 1], sv![2]],
+    ///     sv![OpSize(2), OpSize(3), OpSize(4)],
     /// );
     /// assert_eq!(RearrangeSpec::fuse(&inner, &outer).unwrap(), fused);
     /// ```
@@ -863,9 +878,9 @@ impl RearrangeSpec {
 
         // update inner.inner_ints and outer.inner_ints using the splits we generated above
         fn expand_dims(
-            all_dim_ints: &[Vec<EinInt>],
+            all_dim_ints: &[RInnerInts],
             replaces: &mut HashMap<EinInt, Vec<EinInt>>,
-        ) -> Vec<Vec<EinInt>> {
+        ) -> RInts {
             all_dim_ints
                 .iter()
                 .map(|dim_ints| {
@@ -878,12 +893,12 @@ impl RearrangeSpec {
                 })
                 .collect()
         }
-        let input_ints: Vec<Vec<EinInt>> = expand_dims(&inner.input_ints, &mut inner_replaces);
-        let output_ints: Vec<Vec<EinInt>> = expand_dims(&outer.output_ints, &mut outer_replaces);
+        let input_ints: RInts = expand_dims(&inner.input_ints, &mut inner_replaces);
+        let output_ints: RInts = expand_dims(&outer.output_ints, &mut outer_replaces);
 
         // change inputs to use output labels and drop input terms that don't have a mapping
         // (this should only happen if those indices have size 1)
-        let input_ints: Vec<Vec<EinInt>> = input_ints
+        let input_ints: RInts = input_ints
             .iter()
             .map(|dim_ints| {
                 dim_ints
